package de.fraunhofer.iem.dataprovider.configuration.security

import jakarta.annotation.PostConstruct
import org.springframework.beans.factory.annotation.Value
import org.springframework.http.HttpStatus
import org.springframework.stereotype.Component
import org.springframework.web.server.ServerWebExchange
import org.springframework.web.server.WebFilter
import org.springframework.web.server.WebFilterChain
import reactor.core.publisher.Mono
import java.security.InvalidKeyException

const val API_KEY_HEADER = "Api-Key"
const val API_HEADER_NOT_FOUND_MSG = "Api-Key header not found."

@Component
class ApiKeyFilter : WebFilter {

    @Value("\${API_KEY}")
    lateinit var API_KEY: String

    @PostConstruct
    fun postConstruct() {
        if (API_KEY.length < 30) {
            throw InvalidKeyException("Provided API_KEY is too short. Minimum key length is 30 characters.")
        }
    }

    override fun filter(exchange: ServerWebExchange, chain: WebFilterChain): Mono<Void> {
        val apiKey = exchange.request.headers.getFirst(API_KEY_HEADER)
        return if (apiKey == null) {
            exchange.response.setStatusCode(HttpStatus.BAD_REQUEST)
            val buffer = exchange.response.bufferFactory().wrap(API_HEADER_NOT_FOUND_MSG.encodeToByteArray())
            exchange.response.writeWith(Mono.just(buffer))
        } else {
            if (apiKey != API_KEY) {
                exchange.response.setStatusCode(HttpStatus.UNAUTHORIZED)
                exchange.response.setComplete()
            } else {
                chain.filter(exchange)
            }
        }
    }
}
