package de.fraunhofer.iem.dataprovider.configuration.security

import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.security.config.annotation.web.reactive.EnableWebFluxSecurity
import org.springframework.security.config.web.server.SecurityWebFiltersOrder
import org.springframework.security.config.web.server.ServerHttpSecurity
import org.springframework.security.config.web.server.invoke
import org.springframework.security.web.server.SecurityWebFilterChain


@Configuration
@EnableWebFluxSecurity
class SecurityConfiguration(val apiKeyFilter: ApiKeyFilter) {

    @Bean
    fun springSecurityFilterChain(http: ServerHttpSecurity): SecurityWebFilterChain {
        return http {
            addFilterAt(apiKeyFilter, SecurityWebFiltersOrder.FIRST)
            authorizeExchange {
                authorize("/gitlab/repoChanged", permitAll)
                authorize("/debug/metrics/latestRuns/{platform}/{id}", permitAll)
                authorize("/debug/metrics/recalculateAllKPIs", permitAll)
                authorize("/kpi/{id}", permitAll)
                authorize("/kpi/root/repository/{id}", permitAll)
                authorize("/repository", permitAll)
                authorize("/repository/{id}", permitAll)
            }
            csrf {
                disable()
            }
        }
    }
}
