package de.fraunhofer.iem.dataprovider.repository.entity

import de.fraunhofer.iem.dataprovider.dependency.entity.DependencyEntity
import de.fraunhofer.iem.dataprovider.gitlab.enumeration.PlatformEnum
import de.fraunhofer.iem.dataprovider.tool.entity.ToolEntity
import de.fraunhofer.iem.dataprovider.toolRun.entity.ToolRunEntity
import jakarta.persistence.*
import org.hibernate.annotations.CurrentTimestamp
import org.hibernate.annotations.UpdateTimestamp
import org.hibernate.generator.EventType
import java.time.Instant
import java.util.*

@Entity
@Table(name = "repository")
class RepositoryEntity {

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name = "id", nullable = false)
    var id: UUID? = null

    @OrderBy("last_updated_at DESC")
    @OneToMany(
        mappedBy = "repository",
        cascade = [CascadeType.MERGE, CascadeType.REFRESH, CascadeType.REMOVE, CascadeType.DETACH],
        orphanRemoval = true
    )
    var toolRuns: MutableList<ToolRunEntity> = mutableListOf()

    @Column(name = "name")
    var name: String? = null

    @Column(name = "repo_id")
    var repoId: Long? = null

    @Enumerated
    @Column(name = "platform")
    var platform: PlatformEnum? = null

    @Column(name = "url")
    var url: String? = null

    @CurrentTimestamp(event = [EventType.INSERT])
    var createdAt: Instant? = null

    @UpdateTimestamp
    var lastUpdatedAt: Instant? = null

    @ManyToMany
    @JoinTable(
        name = "repository_tools",
        joinColumns = [JoinColumn(name = "repository_id")],
        inverseJoinColumns = [JoinColumn(name = "tools_id")]
    )
    var toolEntities: MutableSet<ToolEntity> = mutableSetOf()

    @ManyToMany
    @JoinTable(
        name = "repository_dependencies",
        joinColumns = [JoinColumn(name = "repository_id")],
        inverseJoinColumns = [JoinColumn(name = "dependency_id")]
    )
    var dependencyEntities: MutableSet<DependencyEntity> = mutableSetOf()
    fun addToolResults(toolRuns: Collection<ToolRunEntity>) {
        toolRuns.forEach {
            this.addToolResult(it)
        }
    }

    fun addToolResult(toolRun: ToolRunEntity) {
        toolRuns.add(toolRun)
        toolRun.repository = this
    }
}