package de.fraunhofer.iem.dataprovider.taskManager.tasks.tools.sarif.json

import de.fraunhofer.iem.dataprovider.toolRun.entity.FindingEntity
import de.fraunhofer.iem.dataprovider.toolRun.enumeration.LevelEnum
import kotlinx.serialization.SerialName
import kotlinx.serialization.Serializable
import java.util.*

@Serializable
data class SarifResultJson(
    @SerialName("level")
    val level: String,
    @SerialName("locations")
    val locations: List<SarifLocationJson>,
    @SerialName("message")
    val message: SarifMessageJson,
    @SerialName("ruleId")
    val ruleId: String
)

fun SarifResultJson.asDbObject(): FindingEntity {
    val findingEntity = FindingEntity()

    when (this.level.uppercase(Locale.getDefault())) {
        "WARNING" -> findingEntity.level = LevelEnum.Warning
        "ERROR" -> findingEntity.level = LevelEnum.Error
        "NOTE" -> findingEntity.level = LevelEnum.Note
        else -> findingEntity.level = LevelEnum.None
    }

    findingEntity.message = this.message.text
    findingEntity.addLocations(this.locations.map { it.asDbObject() })

    return findingEntity
}
