package de.fraunhofer.iem.dataprovider.tool.entity

import jakarta.persistence.*
import org.hibernate.Hibernate
import org.hibernate.annotations.CurrentTimestamp
import org.hibernate.annotations.UpdateTimestamp
import org.hibernate.tuple.GenerationTiming
import java.time.Instant
import java.util.*

@Entity
@Table(name = "tool")
class ToolEntity {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name = "id", nullable = false)
    var id: UUID? = null

    @Column(name = "full_name", length = 500)
    var fullName: String? = null

    @Column(name = "name", length = 500)
    var name: String? = null

    @Column(name = "version")
    var version: String? = null

    @OneToMany(
        mappedBy = "tool",
        cascade = [CascadeType.MERGE, CascadeType.REFRESH, CascadeType.REMOVE, CascadeType.DETACH],
        orphanRemoval = true
    )
    var ruleEntities: MutableSet<RuleEntity> = mutableSetOf()

    @CurrentTimestamp(timing = GenerationTiming.INSERT)
    var createdAt: Instant? = null

    @UpdateTimestamp
    var lastUpdatedAt: Instant? = null

    fun addRules(ruleEntities: Collection<RuleEntity>) {
        ruleEntities.forEach {
            this.addRule(it)
        }
    }

    fun addRule(ruleEntity: RuleEntity) {
        ruleEntities.add(ruleEntity)
        ruleEntity.tool = this
    }

    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (other == null || Hibernate.getClass(this) != Hibernate.getClass(other)) return false
        other as ToolEntity

        return id != null && id == other.id
    }

    override fun hashCode(): Int = javaClass.hashCode()
}