package de.fraunhofer.iem.dataprovider

import de.fraunhofer.iem.dataprovider.dependency.dto.DependencyCreateDto
import de.fraunhofer.iem.dataprovider.dependency.dto.VulnerabilityCreateDto
import de.fraunhofer.iem.dataprovider.dependency.dto.VulnerabilityScoreCreateDto
import de.fraunhofer.iem.dataprovider.dependency.enumeration.VulnerabilityScoringSystemEnum
import de.fraunhofer.iem.dataprovider.dependency.repository.DependencyRepository
import kotlinx.coroutines.ExperimentalCoroutinesApi
import kotlinx.coroutines.test.runTest
import org.assertj.core.api.Assertions.assertThat
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.fail
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest

@DataJpaTest
class DependencyDatabaseTest @Autowired constructor(
    val dependencyRepository: DependencyRepository
) {

    @OptIn(ExperimentalCoroutinesApi::class)
    @Test
    fun saveDependencyEntityToDb() = runTest {

        val vulnerabilityScores1 = mutableListOf<VulnerabilityScoreCreateDto>()
        vulnerabilityScores1.add(VulnerabilityScoreCreateDto("1.0", VulnerabilityScoringSystemEnum.CVSSV3))
        vulnerabilityScores1.add(VulnerabilityScoreCreateDto("2.0", VulnerabilityScoringSystemEnum.CVSSV3))

        val vulnerabilityScores2 = mutableListOf<VulnerabilityScoreCreateDto>()
        vulnerabilityScores2.add(VulnerabilityScoreCreateDto("3.0", VulnerabilityScoringSystemEnum.CVSSV3))
        vulnerabilityScores2.add(VulnerabilityScoreCreateDto("4.0", VulnerabilityScoringSystemEnum.CVSSV3))


        val vulnerabilities = mutableListOf<VulnerabilityCreateDto>()
        vulnerabilities.add(VulnerabilityCreateDto("MyCVETestIdent1", vulnerabilityScores1))
        vulnerabilities.add(VulnerabilityCreateDto("MyCVETestIdent2", vulnerabilityScores2))

        val createDependencyDependencyCreateDto = DependencyCreateDto(name = "TestDependency", vulnerabilities)

        val dbObject = createDependencyDependencyCreateDto.toDbObject()
        assert(dbObject.name == "TestDependency")
        assert(dbObject.vulnerabilities.size == 2)
        assert(dbObject.vulnerabilities.first().cveIdentifier == "MyCVETestIdent1")
        val savedInstance = dependencyRepository.save(dbObject)
        val queriedInstance =
            dependencyRepository.findById(savedInstance.id!!).orElseGet { fail("Instance was not found in db") }
        assertThat(savedInstance).isEqualTo(queriedInstance)
    }
}